package com.hichina.admin.hichinaadminbackend.mapper;

import com.hichina.admin.hichinaadminbackend.model.DTO.HichinaProductDTO;
import com.hichina.admin.hichinaadminbackend.model.DTO.ProductAttributeTypeShortDTO;
import com.hichina.admin.hichinaadminbackend.model.HichinaLine;
import com.hichina.admin.hichinaadminbackend.model.HichinaProduct;
import com.hichina.admin.hichinaadminbackend.model.ProductAttribute;
import org.apache.ibatis.annotations.*;
import org.springframework.stereotype.Component;

import java.util.List;

@Mapper
@Component(value = "hichinaProductMapper")
public interface HichinaProductMapper {

    @Insert("INSERT INTO hichina_product(sku_id, sku_group_id, product_type_id, product_name, product_content, created_time) VALUES(#{skuId}, #{skuGroupId}, #{productTypeId},#{productName},#{productContent}, #{createdTime})")
    @Options(keyProperty="sku_id")
    void insert(HichinaProduct product);

    @Delete("<script>" +
            "Delete FROM hichina_product WHERE sku_id in \n" +
            "    <foreach item='item' collection='toBeDel' open='(' separator=',' close=')'>\n" +
            "    #{item}" +
            "    </foreach>" +
            "</script>")
    void batchDelete(List<String> toBeDel);

    @Select("select pm.attribute_id, a.attribute_name, a.data_type from hichina_product p inner join product_type_attribute_mapping pm on p.product_type_id=pm.product_type_id inner join product_attribute a on a.attribute_id=pm.attribute_id where p.sku_id=#{skuId}")
    List<ProductAttributeTypeShortDTO> findBindedAttributesBySkuId(String skuId);

    @Select("select sku_id, product_type_name, product_name, product_content, created_time from hichina_product p inner join hichina_product_type t on p.product_type_id=t.product_type_id  where p.product_type_id=#{productTypeId} and product_name like CONCAT('%',CONCAT(#{query},'%')) and product_content like CONCAT('%',CONCAT(#{query},'%')) ")
    List<HichinaProductDTO> findByProductTypeIdAndQuery(String productTypeId, String query);

    @Select("select sku_id, product_type_name, product_name, product_content, created_time from hichina_product p inner join hichina_product_type t on p.product_type_id=t.product_type_id  ")
    List<HichinaProductDTO> findAllProduct();


    @Select("select sku_id, product_type_name, product_name, product_content, created_time from hichina_product p inner join hichina_product_type t on p.product_type_id=t.product_type_id  where product_name like CONCAT('%',CONCAT(#{query},'%')) and product_content like CONCAT('%',CONCAT(#{query},'%')) ")
    List<HichinaProductDTO> findByQuery(String query);

    @Select("select sku_id, product_type_name, product_name, product_content, created_time from hichina_product p inner join hichina_product_type t on p.product_type_id=t.product_type_id  where p.product_type_id=#{productTypeId} ")
    List<HichinaProductDTO> findByProductTypeId(String productTypeId);

    @Select("select sku_id, product_type_name, product_name, product_content, created_time from hichina_product p inner join hichina_product_type t on p.product_type_id=t.product_type_id  where p.sku_id=#{skuId} ")
    List<HichinaProductDTO> findBySkuId(String skuId);

    @Select("select * from hichina_product where sku_id=#{skuId}")
    List<HichinaProduct> findRawBySkuId(String skuId);


    @Select("select count(*) from hichina_product where product_type_id=#{productTypeId} and product_name like CONCAT('%',CONCAT(#{query},'%')) and product_content like CONCAT('%',CONCAT(#{query},'%')) ")
    int countByProductTypeIdAndQuery(String productTypeId, String query);

    @Select("select count(*) from hichina_product where product_name like CONCAT('%',CONCAT(#{query},'%')) and product_content like CONCAT('%',CONCAT(#{query},'%')) ")
    int countByQuery(String query);

    @Update("update hichina_product SET product_type_id=#{productTypeId}, sku_group_id=#{skuGroupId}, product_name=#{productName}, product_content=#{productContent}, created_time=#{createdTime} where sku_id=#{skuId}")
    void updateBySkuId(HichinaProduct product);

    @Select("select count(*) from hichina_product where product_type_id=#{productTypeId}")
    int countByProductTypeId(String productTypeId);

    @Select("select count(*) from hichina_product")
    int count();


}
