package com.hichina.admin.hichinaadminbackend.mapper;

import com.hichina.admin.hichinaadminbackend.model.DTO.SkuGroupStatsDTO;
import com.hichina.admin.hichinaadminbackend.model.HichinaLine;
import com.hichina.admin.hichinaadminbackend.model.ProductAttribute;
import com.hichina.admin.hichinaadminbackend.model.ProductSkuGroup;
import org.apache.ibatis.annotations.*;
import org.springframework.stereotype.Component;

import java.util.List;

@Mapper
@Component(value = "productSkuGroupMapper")
public interface ProductSkuGroupMapper {
    @Insert("INSERT INTO product_sku_group(sku_group_id, sku_group_name, product_type_id, created_date) VALUES(#{skuGroupId},#{skuGroupName}, #{productTypeId}, #{createdDate})")
    void insert(ProductSkuGroup productSkuGroup);

    @Update("update product_sku_group SET sku_group_name=#{skuGroupName}, product_type_id=#{productTypeId}, created_date=#{createdDate}, enabled=#{enabled} where sku_group_id=#{skuGroupId}")
    void update(ProductSkuGroup group);

    @Select("select * from product_sku_group where sku_group_name=#{name} and product_type_id=#{productTypeId}")
    List<ProductSkuGroup> findByNameAndProductTypeId(String name, String productTypeId);

    @Select("select * from product_sku_group where sku_group_id=#{skuGroupId}")
    List<ProductSkuGroup> findBySkuGroupId(String skuGroupId);

    @Select("select temp.sku_group_id, temp.sku_group_name, temp.product_type_id, t.product_type_name, temp.product_count, temp.enabled  from (select g.sku_group_id , g.sku_group_name , g.product_type_id , g.enabled, count(p.sku_id) as product_count  from product_sku_group g inner join hichina_product p on g.sku_group_id = p.sku_group_id group by g.sku_group_id , g.sku_group_name , g.product_type_id order by g.created_date desc) temp inner join hichina_product_type t on t.product_type_id = temp.product_type_id")
    List<SkuGroupStatsDTO> findAllSkuGroupStats();

    @Select("select count(*) from (select g.sku_group_id , g.sku_group_name , g.product_type_id , count(p.sku_id) as product_count  from product_sku_group g inner join hichina_product p on g.sku_group_id = p.sku_group_id group by g.sku_group_id , g.sku_group_name , g.product_type_id order by g.created_date desc) temp inner join hichina_product_type t on t.product_type_id = temp.product_type_id")
    Integer countAllSkuGroupStats();

    @Select("select temp.sku_group_id, temp.sku_group_name, temp.product_type_id, t.product_type_name, temp.product_count, temp.enabled  from (select g.sku_group_id , g.sku_group_name , g.product_type_id , g.enabled, count(p.sku_id) as product_count  from product_sku_group g inner join hichina_product p on g.sku_group_id = p.sku_group_id group by g.sku_group_id , g.sku_group_name , g.product_type_id order by g.created_date desc) temp inner join hichina_product_type t on t.product_type_id = temp.product_type_id where temp.sku_group_name like CONCAT('%',CONCAT(#{query},'%'))")
    List<SkuGroupStatsDTO> findSkuGroupStatsByQuery(String query);

    @Select("select count(*)  from (select g.sku_group_id , g.sku_group_name , g.product_type_id , count(p.sku_id) as product_count  from product_sku_group g inner join hichina_product p on g.sku_group_id = p.sku_group_id group by g.sku_group_id , g.sku_group_name , g.product_type_id order by g.created_date desc) temp inner join hichina_product_type t on t.product_type_id = temp.product_type_id where temp.sku_group_name like CONCAT('%',CONCAT(#{query},'%'))")
    Integer countSkuGroupStatsByQuery(String query);

    @Delete("delete from product_sku_group where sku_group_id not in (select distinct sku_group_id from hichina_product)")
    void deleteUnreferenced();
}
